@@ -249,8 +249,7 @@ func (dcs *DoltChunkStore) GetManyCompressed(ctx context.Context, hashes hash.Ha
 }
 
 const (
-	getLocsBatchSize      = 32 * 1024
-	getLocsMaxConcurrency = 4
+	getLocsBatchSize = 256
 )
 
 type GetRange remotesapi.HttpGetRange
@@ -324,12 +323,24 @@ func (gr *GetRange) ChunkByteRange(i int) (uint64, uint64) {
 	return start, end
 }
 
-func (gr *GetRange) GetDownloadFunc(ctx context.Context, fetcher HTTPFetcher, chunkChan chan nbs.CompressedChunk) func() error {
+type resourcePathToUrlFunc func(ctx context.Context, resourcePath string) (url string, err error)
+
+func (gr *GetRange) GetDownloadFunc(ctx context.Context, fetcher HTTPFetcher, chunkChan chan nbs.CompressedChunk, pathToUrl resourcePathToUrlFunc) func() error {
 	if len(gr.Ranges) == 0 {
 		return func() error { return nil }
 	}
 	return func() error {
-		comprData, err := hedgedRangeDownloadWithRetries(ctx, fetcher, gr.ChunkStartOffset(0), gr.RangeLen(), gr.Url)
+		urlF := func() (string, error) {
+			url, err := pathToUrl(ctx, gr.ResourcePath())
+			if err != nil {
+				return "", err
+			}
+			if url == "" {
+				url = gr.Url
+			}
+			return url, nil
+		}
+		comprData, err := hedgedRangeDownloadWithRetries(ctx, fetcher, gr.ChunkStartOffset(0), gr.RangeLen(), urlF)
 		if err != nil {
 			return err
 		}
@@ -350,11 +361,77 @@ func (gr *GetRange) GetDownloadFunc(ctx context.Context, fetcher HTTPFetcher, ch
 	}
 }
 
-func (dcs *DoltChunkStore) getDLLocs(ctx context.Context, hashes []hash.Hash) (map[string]*GetRange, error) {
-	res := make(map[string]*GetRange)
+type locationRefresh struct {
+	RefreshAfter   time.Time
+	RefreshRequest *remotesapi.RefreshTableFileUrlRequest
+	URL            string
+	mu             *sync.Mutex
+}
+
+func (r *locationRefresh) Add(resp *remotesapi.DownloadLoc) {
+	if r.URL == "" {
+		r.URL = resp.Location.(*remotesapi.DownloadLoc_HttpGetRange).HttpGetRange.Url
+	}
+	if resp.RefreshAfter == nil {
+		return
+	}
+	respTime := resp.RefreshAfter.AsTime()
+	if (r.RefreshAfter == time.Time{}) || respTime.After(r.RefreshAfter) {
+		r.RefreshAfter = resp.RefreshAfter.AsTime()
+		r.RefreshRequest = resp.RefreshRequest
+		r.URL = resp.Location.(*remotesapi.DownloadLoc_HttpGetRange).HttpGetRange.Url
+	}
+}
+
+func (r *locationRefresh) GetURL(ctx context.Context, client remotesapi.ChunkStoreServiceClient) (string, error) {
+	r.mu.Lock()
+	defer r.mu.Unlock()
+	if r.RefreshRequest != nil && time.Now().After(r.RefreshAfter) {
+		resp, err := client.RefreshTableFileUrl(ctx, r.RefreshRequest)
+		if err != nil {
+			return r.URL, err
+		}
+		r.RefreshAfter = resp.RefreshAfter.AsTime()
+		r.URL = resp.Url
+	}
+	return r.URL, nil
+}
+
+type dlLocations struct {
+	ranges    map[string]*GetRange
+	refreshes map[string]*locationRefresh
+}
+
+func newDlLocations() dlLocations {
+	return dlLocations{
+		ranges:    make(map[string]*GetRange),
+		refreshes: make(map[string]*locationRefresh),
+	}
+}
+
+func (l *dlLocations) Add(resp *remotesapi.DownloadLoc) {
+	gr := (*GetRange)(resp.Location.(*remotesapi.DownloadLoc_HttpGetRange).HttpGetRange)
+	path := gr.ResourcePath()
+	if v, ok := l.ranges[path]; ok {
+		v.Append(gr)
+		l.refreshes[path].Add(resp)
+	} else {
+		l.ranges[path] = gr
+		refresh := &locationRefresh{mu: new(sync.Mutex)}
+		refresh.Add(resp)
+		l.refreshes[path] = refresh
+	}
+}
+
+func (dcs *DoltChunkStore) getDLLocs(ctx context.Context, hashes []hash.Hash) (dlLocations, error) {
+	span, ctx := tracing.StartSpan(ctx, "remotestorage.getDLLocs")
+	span.LogKV("num_hashes", len(hashes))
+	defer span.Finish()
+
+	res := newDlLocations()
 
 	// channel for receiving results from go routines making grpc calls to get download locations for chunks
-	dlLocChan := make(chan []*remotesapi.HttpGetRange)
+	resCh := make(chan []*remotesapi.DownloadLoc)
 
 	eg, ctx := errgroup.WithContext(ctx)
 
@@ -362,17 +439,12 @@ func (dcs *DoltChunkStore) getDLLocs(ctx context.Context, hashes []hash.Hash) (m
 	eg.Go(func() error {
 		for {
 			select {
-			case locs, ok := <-dlLocChan:
+			case locs, ok := <-resCh:
 				if !ok {
 					return nil
 				}
 				for _, loc := range locs {
-					gr := (*GetRange)(loc)
-					if v, ok := res[gr.ResourcePath()]; ok {
-						v.Append(gr)
-					} else {
-						res[gr.ResourcePath()] = gr
-					}
+					res.Add(loc)
 				}
 			case <-ctx.Done():
 				return ctx.Err()
@@ -380,54 +452,69 @@ func (dcs *DoltChunkStore) getDLLocs(ctx context.Context, hashes []hash.Hash) (m
 		}
 	})
 
-	hashesBytes := HashesToSlices(hashes)
-	var work []func() error
-
-	// batchItr creates work functions which request a batch of chunk download locations and write the results to the
-	// dlLocChan
-	batchItr(len(hashesBytes), getLocsBatchSize, func(st, end int) (stop bool) {
-		batch := hashesBytes[st:end]
-		f := func() error {
-			req := &remotesapi.GetDownloadLocsRequest{RepoId: dcs.getRepoId(), ChunkHashes: batch}
-			resp, err := dcs.csClient.GetDownloadLocations(ctx, req)
-			if err != nil {
-				return NewRpcError(err, "GetDownloadLocations", dcs.host, req)
-			}
-			tosend := make([]*remotesapi.HttpGetRange, len(resp.Locs))
-			for i, l := range resp.Locs {
-				tosend[i] = l.Location.(*remotesapi.DownloadLoc_HttpGetRange).HttpGetRange
-			}
-			select {
-			case dlLocChan <- tosend:
-			case <-ctx.Done():
-			}
-			return nil
-		}
-		work = append(work, f)
-		return false
-	})
-
-	span, ctx := tracing.StartSpan(ctx, "remotestorage.getDLLocs")
-	span.LogKV("num_batches", len(work), "num_hashes", len(hashes))
-	defer span.Finish()
-
-	// execute the work and close the channel after as no more results will come in
+	// go routine for batching the get location requests, streaming the requests and streaming the responses.
 	eg.Go(func() error {
-		defer close(dlLocChan)
-		return concurrentExec(work, getLocsMaxConcurrency)
+		var reqs []*remotesapi.GetDownloadLocsRequest
+		hashesBytes := HashesToSlices(hashes)
+		batchItr(len(hashesBytes), getLocsBatchSize, func(st, end int) (stop bool) {
+			batch := hashesBytes[st:end]
+			req := &remotesapi.GetDownloadLocsRequest{RepoId: dcs.getRepoId(), ChunkHashes: batch}
+			reqs = append(reqs, req)
+			return false
+		})
+		op := func() error {
+			seg, ctx := errgroup.WithContext(ctx)
+			stream, err := dcs.csClient.StreamDownloadLocations(ctx)
+			if err != nil {
+				return NewRpcError(err, "StreamDownloadLocations", dcs.host, nil)
+			}
+			completedReqs := 0
+			// Write requests
+			seg.Go(func() error {
+				for i := range reqs {
+					if err := stream.Send(reqs[i]); err != nil {
+						return NewRpcError(err, "StreamDownloadLocations", dcs.host, reqs[i])
+					}
+				}
+				return stream.CloseSend()
+			})
+			// Read responses
+			seg.Go(func() error {
+				for {
+					resp, err := stream.Recv()
+					if err != nil {
+						if err == io.EOF {
+							return nil
+						}
+						return NewRpcError(err, "StreamDownloadLocations", dcs.host, reqs[completedReqs])
+					}
+					select {
+					case resCh <- resp.Locs:
+						completedReqs += 1
+					case <-ctx.Done():
+						return ctx.Err()
+					}
+				}
+			})
+			err = seg.Wait()
+			reqs = reqs[completedReqs:]
+			if len(reqs) == 0 {
+				close(resCh)
+			}
+			return processGrpcErr(err)
+		}
+		return backoff.Retry(op, backoff.WithMaxRetries(csRetryParams, csClientRetries))
 	})
 
 	if err := eg.Wait(); err != nil {
-		return nil, err
+		return dlLocations{}, err
 	}
-
 	return res, nil
 }
 
 func (dcs *DoltChunkStore) readChunksAndCache(ctx context.Context, hashes hash.HashSet, notCached []hash.Hash, found func(nbs.CompressedChunk)) error {
 	// get the locations where the chunks can be downloaded from
-	resourceToUrlAndRanges, err := dcs.getDLLocs(ctx, notCached)
-
+	dlLocs, err := dcs.getDLLocs(ctx, notCached)
 	if err != nil {
 		return err
 	}
@@ -460,7 +547,7 @@ func (dcs *DoltChunkStore) readChunksAndCache(ctx context.Context, hashes hash.H
 	// download the chunks and close the channel after
 	func() {
 		defer close(chunkChan)
-		err = dcs.downloadChunks(ctx, resourceToUrlAndRanges, chunkChan)
+		err = dcs.downloadChunks(ctx, dlLocs, chunkChan)
 	}()
 
 	// wait for all the results to finish processing
@@ -835,17 +922,23 @@ func logDownloadStats(span opentracing.Span, originalGets map[string]*GetRange,
 
 // creates work functions for each download and executes them in parallel.  The work functions write downloaded chunks
 // to chunkChan
-func (dcs *DoltChunkStore) downloadChunks(ctx context.Context, resourceGets map[string]*GetRange, chunkChan chan nbs.CompressedChunk) error {
+func (dcs *DoltChunkStore) downloadChunks(ctx context.Context, dlLocs dlLocations, chunkChan chan nbs.CompressedChunk) error {
 	span, ctx := tracing.StartSpan(ctx, "remotestorage.downloadChunks")
 	defer span.Finish()
 
+	resourceGets := dlLocs.ranges
+
 	gets := aggregateDownloads(chunkAggDistance, resourceGets)
 	logDownloadStats(span, resourceGets, gets)
 
+	toUrl := func(ctx context.Context, resourcePath string) (string, error) {
+		return dlLocs.refreshes[resourcePath].GetURL(ctx, dcs.csClient)
+	}
+
 	// loop over all the gets that need to be downloaded and create a work function for each
 	work := make([]func() error, len(gets))
 	for i, get := range gets {
-		work[i] = get.GetDownloadFunc(ctx, dcs.httpFetcher, chunkChan)
+		work[i] = get.GetDownloadFunc(ctx, dcs.httpFetcher, chunkChan, toUrl)
 	}
 
 	// execute the work
@@ -854,10 +947,10 @@ func (dcs *DoltChunkStore) downloadChunks(ctx context.Context, resourceGets map[
 	return err
 }
 
-func hedgedRangeDownloadWithRetries(ctx context.Context, fetcher HTTPFetcher, offset, length uint64, urlStr string) ([]byte, error) {
+func hedgedRangeDownloadWithRetries(ctx context.Context, fetcher HTTPFetcher, offset, length uint64, urlStrF func() (string, error)) ([]byte, error) {
 	res, err := DownloadHedger.Do(ctx, Work{
 		Work: func(ctx context.Context) (interface{}, error) {
-			return rangeDownloadWithRetries(ctx, fetcher, offset, length, urlStr)
+			return rangeDownloadWithRetries(ctx, fetcher, offset, length, urlStrF)
 		},
 		Size: int(length),
 	})
@@ -869,12 +962,8 @@ func hedgedRangeDownloadWithRetries(ctx context.Context, fetcher HTTPFetcher, of
 
 // rangeDownloadWithRetries executes an http get with the 'Range' header to get a range of bytes from a file.  Request
 // is executed with retries and if progress was made, downloads will be resumed from where they left off on subsequent attempts.
-func rangeDownloadWithRetries(ctx context.Context, fetcher HTTPFetcher, offset, length uint64, urlStr string) ([]byte, error) {
+func rangeDownloadWithRetries(ctx context.Context, fetcher HTTPFetcher, offset, length uint64, urlStrF func() (string, error)) ([]byte, error) {
 	// create the request
-	req, err := http.NewRequest(http.MethodGet, urlStr, nil)
-	if err != nil {
-		return nil, err
-	}
 
 	// parameters used for resuming downloads.
 	var allBufs [][]byte
@@ -883,6 +972,16 @@ func rangeDownloadWithRetries(ctx context.Context, fetcher HTTPFetcher, offset,
 
 	//execute the request
 	op := func() error {
+		urlStr, err := urlStrF()
+		if err != nil {
+			return err
+		}
+
+		req, err := http.NewRequest(http.MethodGet, urlStr, nil)
+		if err != nil {
+			return err
+		}
+
 		rangeVal := fmt.Sprintf("bytes=%d-%d", currOffset, currOffset+currLength-1)
 		req.Header.Set("Range", rangeVal)
 
@@ -910,8 +1009,7 @@ func rangeDownloadWithRetries(ctx context.Context, fetcher HTTPFetcher, offset,
 		return err
 	}
 
-	err = backoff.Retry(op, backoff.WithMaxRetries(downRetryParams, downRetryCount))
-
+	err := backoff.Retry(op, backoff.WithMaxRetries(downRetryParams, downRetryCount))
 	if err != nil {
 		return nil, err
 	}
@@ -1067,6 +1165,14 @@ func sanitizeSignedUrl(url string) string {
 
 // Open returns an io.ReadCloser which can be used to read the bytes of a table file.
 func (drtf DoltRemoteTableFile) Open(ctx context.Context) (io.ReadCloser, error) {
+	if drtf.info.RefreshAfter != nil && drtf.info.RefreshAfter.AsTime().After(time.Now()) {
+		resp, err := drtf.dcs.csClient.RefreshTableFileUrl(ctx, drtf.info.RefreshRequest)
+		if err == nil {
+			drtf.info.Url = resp.Url
+			drtf.info.RefreshAfter = resp.RefreshAfter
+		}
+	}
+
 	req, err := http.NewRequestWithContext(ctx, http.MethodGet, drtf.info.Url, nil)
 	if err != nil {
 		return nil, err
