@@ -73,7 +73,7 @@ func (d DisabledTransaction) String() string {
 	return "Disabled transaction"
 }
 
-func (db Database) BeginTransaction(ctx *sql.Context) (sql.Transaction, error) {
+func (db Database) StartTransaction(ctx *sql.Context) (sql.Transaction, error) {
 	if !transactionsEnabled {
 		return DisabledTransaction{}, nil
 	}
@@ -104,23 +104,27 @@ func (db Database) BeginTransaction(ctx *sql.Context) (sql.Transaction, error) {
 
 func (db Database) CommitTransaction(ctx *sql.Context, tx sql.Transaction) error {
 	dsession := DSessFromSess(ctx.Session)
-	return dsession.CommitTransaction(ctx, db.name)
+	return dsession.CommitTransaction(ctx, db.name, tx)
 }
 
-func (db Database) Rollback(ctx *sql.Context, transaction sql.Transaction) error {
-	return nil
+func (db Database) Rollback(ctx *sql.Context, tx sql.Transaction) error {
+	dsession := DSessFromSess(ctx.Session)
+	return dsession.RollbackTransaction(ctx, db.name, tx)
 }
 
-func (db Database) CreateSavepoint(ctx *sql.Context, transaction sql.Transaction, name string) error {
-	return nil
+func (db Database) CreateSavepoint(ctx *sql.Context, tx sql.Transaction, name string) error {
+	dsession := DSessFromSess(ctx.Session)
+	return dsession.CreateSavepoint(ctx, name, db.name, tx)
 }
 
-func (db Database) RollbackToSavepoint(ctx *sql.Context, transaction sql.Transaction, name string) error {
-	return nil
+func (db Database) RollbackToSavepoint(ctx *sql.Context, tx sql.Transaction, name string) error {
+	dsession := DSessFromSess(ctx.Session)
+	return dsession.RollbackToSavepoint(ctx, name, db.name, tx)
 }
 
-func (db Database) ReleaseSavepoint(ctx *sql.Context, transaction sql.Transaction, name string) error {
-	return nil
+func (db Database) ReleaseSavepoint(ctx *sql.Context, tx sql.Transaction, name string) error {
+	dsession := DSessFromSess(ctx.Session)
+	return dsession.ReleaseSavepoint(ctx, name, db.name, tx)
 }
 
 var _ SqlDatabase = Database{}
@@ -181,6 +185,15 @@ func (db Database) GetDocsReadWriter() env.DocsReadWriter {
 	return db.drw
 }
 
+func (db Database) DbData() env.DbData {
+	return env.DbData{
+		Ddb: db.ddb,
+		Rsw: db.rsw,
+		Rsr: db.rsr,
+		Drw: db.drw,
+	}
+}
+
 // GetTableInsensitive is used when resolving tables in queries. It returns a best-effort case-insensitive match for
 // the table name given.
 func (db Database) GetTableInsensitive(ctx *sql.Context, tblName string) (sql.Table, bool, error) {
@@ -258,7 +271,24 @@ func (db Database) GetTableInsensitiveAsOf(ctx *sql.Context, tableName string, a
 		return nil, false, nil
 	}
 
-	return db.getTable(ctx, root, tableName)
+	table, ok, err := db.getTable(ctx, root, tableName)
+	if err != nil {
+		return nil, false, err
+	}
+	if !ok {
+		return nil, false, nil
+	}
+
+	switch table := table.(type) {
+	case *DoltTable:
+		return table.LockedToRoot(root), true, nil
+	case *AlterableDoltTable:
+		return table.LockedToRoot(root), true, nil
+	case *WritableDoltTable:
+		return table.LockedToRoot(root), true, nil
+	default:
+		panic(fmt.Sprintf("unexpected table type %T", table))
+	}
 }
 
 // rootAsOf returns the root of the DB as of the expression given, which may be nil in the case that it refers to an
@@ -354,11 +384,6 @@ func (db Database) GetTableNamesAsOf(ctx *sql.Context, time interface{}) ([]stri
 // getTable gets the table with the exact name given at the root value given. The database caches tables for all root
 // values to avoid doing schema lookups on every table lookup, which are expensive.
 func (db Database) getTable(ctx *sql.Context, root *doltdb.RootValue, tableName string) (sql.Table, bool, error) {
-	cache := TableCacheFromSess(ctx.Session, db.name)
-	if table, ok := cache.Get(tableName, root); ok {
-		return table, true, nil
-	}
-
 	tableNames, err := getAllTableNames(ctx, root)
 	if err != nil {
 		return nil, true, err
@@ -386,15 +411,13 @@ func (db Database) getTable(ctx *sql.Context, root *doltdb.RootValue, tableName
 
 	readonlyTable := NewDoltTable(tableName, sch, tbl, db)
 	if doltdb.IsReadOnlySystemTable(tableName) {
-		table = &readonlyTable
+		table = readonlyTable
 	} else if doltdb.HasDoltPrefix(tableName) {
 		table = &WritableDoltTable{DoltTable: readonlyTable, db: db}
 	} else {
 		table = &AlterableDoltTable{WritableDoltTable{DoltTable: readonlyTable, db: db}}
 	}
 
-	cache.Put(tableName, root, table)
-
 	return table, true, nil
 }
 
@@ -481,6 +504,15 @@ func (db Database) LoadRootFromRepoState(ctx *sql.Context) error {
 	return db.SetRoot(ctx, root)
 }
 
+func (db Database) GetHeadRoot(ctx *sql.Context) (*doltdb.RootValue, error) {
+	dsess := DSessFromSess(ctx.Session)
+	head, _, err := dsess.GetHeadCommit(ctx, db.name)
+	if err != nil {
+		return nil, err
+	}
+	return head.GetRootValue()
+}
+
 // DropTable drops the table with the name given
 func (db Database) DropTable(ctx *sql.Context, tableName string) error {
 	root, err := db.GetRoot(ctx)
@@ -536,7 +568,12 @@ func (db Database) createSqlTable(ctx *sql.Context, tableName string, sch sql.Sc
 		return sql.ErrTableAlreadyExists.New(tableName)
 	}
 
-	doltSch, err := sqlutil.ToDoltSchema(ctx, root, tableName, sch)
+	headRoot, err := db.GetHeadRoot(ctx)
+	if err != nil {
+		return err
+	}
+
+	doltSch, err := sqlutil.ToDoltSchema(ctx, root, tableName, sch, headRoot)
 	if err != nil {
 		return err
 	}
@@ -558,7 +595,7 @@ func (db Database) createDoltTable(ctx *sql.Context, tableName string, root *dol
 		if err != nil {
 			return true, err
 		}
-		if exists {
+		if exists && tbl != tableName {
 			errStr := schema.ErrTagPrevUsed(tag, col.Name, tbl).Error()
 			conflictingTbls = append(conflictingTbls, errStr)
 		}
@@ -612,33 +649,15 @@ func (db Database) RenameTable(ctx *sql.Context, oldName, newName string) error
 
 // Flush flushes the current batch of outstanding changes and returns any errors.
 func (db Database) Flush(ctx *sql.Context) error {
-	root, err := db.GetRoot(ctx)
+	dsess := DSessFromSess(ctx.Session)
+	editSession := dsess.editSessions[db.name]
+
+	newRoot, err := editSession.Flush(ctx)
 	if err != nil {
 		return err
 	}
 
-	cache := TableCacheFromSess(ctx.Session, db.name)
-	tables, ok := cache.AllForRoot(root)
-
-	if ok {
-		for _, table := range tables {
-			if writable, ok := table.(*WritableDoltTable); ok {
-				if err := writable.flushBatchedEdits(ctx); err != nil {
-					return err
-				}
-			} else if alterable, ok := table.(*AlterableDoltTable); ok {
-				if err := alterable.flushBatchedEdits(ctx); err != nil {
-					return err
-				}
-			} else if writable, ok := table.(*WritableIndexedDoltTable); ok {
-				if err := writable.flushBatchedEdits(ctx); err != nil {
-					return err
-				}
-			}
-		}
-	}
-
-	return nil
+	return db.SetRoot(ctx, newRoot)
 }
 
 // CreateView implements sql.ViewCreator. Persists the view in the dolt database, so
@@ -657,35 +676,44 @@ func (db Database) DropView(ctx *sql.Context, name string) error {
 
 // GetTriggers implements sql.TriggerDatabase.
 func (db Database) GetTriggers(ctx *sql.Context) ([]sql.TriggerDefinition, error) {
-	sqlTbl, ok, err := db.GetTableInsensitive(ctx, doltdb.SchemasTableName)
+	root, err := db.GetRoot(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	tbl, ok, err := root.GetTable(ctx, doltdb.SchemasTableName)
 	if err != nil {
 		return nil, err
 	}
 	if !ok {
 		return nil, nil
 	}
-	tbl := sqlTbl.(*WritableDoltTable)
 
-	typeCol, ok := tbl.sch.GetAllCols().GetByName(doltdb.SchemasTablesTypeCol)
+	sch, err := tbl.GetSchema(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	typeCol, ok := sch.GetAllCols().GetByName(doltdb.SchemasTablesTypeCol)
 	if !ok {
 		return nil, fmt.Errorf("`%s` schema in unexpected format", doltdb.SchemasTableName)
 	}
-	nameCol, ok := tbl.sch.GetAllCols().GetByName(doltdb.SchemasTablesNameCol)
+	nameCol, ok := sch.GetAllCols().GetByName(doltdb.SchemasTablesNameCol)
 	if !ok {
 		return nil, fmt.Errorf("`%s` schema in unexpected format", doltdb.SchemasTableName)
 	}
-	fragCol, ok := tbl.sch.GetAllCols().GetByName(doltdb.SchemasTablesFragmentCol)
+	fragCol, ok := sch.GetAllCols().GetByName(doltdb.SchemasTablesFragmentCol)
 	if !ok {
 		return nil, fmt.Errorf("`%s` schema in unexpected format", doltdb.SchemasTableName)
 	}
 
-	rowData, err := tbl.table.GetRowData(ctx)
+	rowData, err := tbl.GetRowData(ctx)
 	if err != nil {
 		return nil, err
 	}
 	var triggers []sql.TriggerDefinition
 	err = rowData.Iter(ctx, func(key, val types.Value) (stop bool, err error) {
-		dRow, err := row.FromNoms(tbl.sch, key.(types.Tuple), val.(types.Tuple))
+		dRow, err := row.FromNoms(sch, key.(types.Tuple), val.(types.Tuple))
 		if err != nil {
 			return true, err
 		}
@@ -733,22 +761,32 @@ func (db Database) DropTrigger(ctx *sql.Context, name string) error {
 func (db Database) GetStoredProcedures(ctx *sql.Context) ([]sql.StoredProcedureDetails, error) {
 	missingValue := errors.NewKind("missing `%s` value for procedure row: (%s)")
 
-	sqlTbl, ok, err := db.GetTableInsensitive(ctx, doltdb.ProceduresTableName)
+	root, err := db.GetRoot(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	table, ok, err := root.GetTable(ctx, doltdb.ProceduresTableName)
 	if err != nil {
 		return nil, err
 	}
 	if !ok {
 		return nil, nil
 	}
-	tbl := sqlTbl.(*WritableDoltTable)
 
-	rowData, err := tbl.table.GetRowData(ctx)
+	rowData, err := table.GetRowData(ctx)
 	if err != nil {
 		return nil, err
 	}
+
+	sch, err := table.GetSchema(ctx)
+	if err != nil {
+		return nil, err
+	}
+
 	var spds []sql.StoredProcedureDetails
 	err = rowData.Iter(ctx, func(key, val types.Value) (stop bool, err error) {
-		dRow, err := row.FromNoms(tbl.sch, key.(types.Tuple), val.(types.Tuple))
+		dRow, err := row.FromNoms(sch, key.(types.Tuple), val.(types.Tuple))
 		if err != nil {
 			return true, err
 		}
@@ -887,7 +925,12 @@ func (db Database) TableEditSession(ctx *sql.Context) *editor.TableEditSession {
 // schema fragments if they don't parse, or if registries within the
 // `catalog` return errors.
 func RegisterSchemaFragments(ctx *sql.Context, db Database, root *doltdb.RootValue) error {
-	stbl, found, err := db.GetTableInsensitiveWithRoot(ctx, root, doltdb.SchemasTableName)
+	root, err := db.GetRoot(ctx)
+	if err != nil {
+		return err
+	}
+
+	tbl, found, err := root.GetTable(ctx, doltdb.SchemasTableName)
 	if err != nil {
 		return err
 	}
@@ -895,14 +938,13 @@ func RegisterSchemaFragments(ctx *sql.Context, db Database, root *doltdb.RootVal
 		return nil
 	}
 
-	tbl := stbl.(*WritableDoltTable)
-	rowData, err := tbl.table.GetRowData(ctx)
+	rowData, err := tbl.GetRowData(ctx)
 
 	if err != nil {
 		return err
 	}
 
-	iter, err := newRowIterator(ctx, &tbl.DoltTable, nil, &doltTablePartition{rowData: rowData, end: NoUpperBound})
+	iter, err := newRowIterator(ctx, tbl, nil, &doltTablePartition{rowData: rowData, end: NoUpperBound})
 	if err != nil {
 		return err
 	}
