@@ -77,7 +77,6 @@ type DoltSession struct {
 	dbDatas      map[string]env.DbData
 	editSessions map[string]*editor.TableEditSession
 	dirty        map[string]bool
-	caches       map[string]TableCache
 	Username     string
 	Email        string
 }
@@ -92,7 +91,6 @@ func DefaultDoltSession() *DoltSession {
 		dbDatas:      make(map[string]env.DbData),
 		editSessions: make(map[string]*editor.TableEditSession),
 		dirty:        make(map[string]bool),
-		caches:       make(map[string]TableCache),
 		workingSets:  make(map[string]ref.WorkingSetRef),
 		Username:     "",
 		Email:        "",
@@ -117,12 +115,11 @@ func NewDoltSession(ctx *sql.Context, sqlSess sql.Session, username, email strin
 		dirty:        make(map[string]bool),
 		roots:        make(map[string]dbRoot),
 		workingSets:  make(map[string]ref.WorkingSetRef),
-		caches:       make(map[string]TableCache),
 		Username:     username,
 		Email:        email,
 	}
 	for _, db := range dbs {
-		err := sess.AddDB(ctx, db)
+		err := sess.AddDB(ctx, db, db.DbData())
 
 		if err != nil {
 			return nil, err
@@ -137,12 +134,8 @@ func DSessFromSess(sess sql.Session) *DoltSession {
 	return sess.(*DoltSession)
 }
 
-func TableCacheFromSess(sess sql.Session, dbName string) TableCache {
-	return sess.(*DoltSession).caches[dbName]
-}
-
 // CommitTransaction commits the in-progress transaction for the database named
-func (sess *DoltSession) CommitTransaction(ctx *sql.Context, dbName string) error {
+func (sess *DoltSession) CommitTransaction(ctx *sql.Context, dbName string, tx sql.Transaction) error {
 	if !sess.dirty[dbName] {
 		return nil
 	}
@@ -176,11 +169,6 @@ func (sess *DoltSession) CommitTransaction(ctx *sql.Context, dbName string) erro
 
 	// Newer commit path does a concurrent merge of the current root with the one other clients are editing, then
 	// updates the session with this new root.
-	tx := ctx.GetTransaction()
-	if tx == nil {
-		return nil
-	}
-
 	// TODO: validate that the transaction belongs to the DB named
 	dtx, ok := tx.(*DoltTransaction)
 	if !ok {
@@ -201,6 +189,85 @@ func (sess *DoltSession) CommitTransaction(ctx *sql.Context, dbName string) erro
 	return nil
 }
 
+// RollbackTransaction rolls the given transaction back
+func (sess *DoltSession) RollbackTransaction(ctx *sql.Context, dbName string, tx sql.Transaction) error {
+	if !transactionsEnabled || dbName == "" {
+		return nil
+	}
+
+	if !sess.dirty[dbName] {
+		return nil
+	}
+
+	dtx, ok := tx.(*DoltTransaction)
+	if !ok {
+		return fmt.Errorf("expected a DoltTransaction")
+	}
+
+	err := sess.SetRoot(ctx, dbName, dtx.startRoot)
+	if err != nil {
+		return err
+	}
+
+	sess.dirty[dbName] = false
+	return nil
+}
+
+// CreateSavepoint creates a new savepoint for this transaction with the name given. A previously created savepoint
+// with the same name will be overwritten.
+func (sess *DoltSession) CreateSavepoint(ctx *sql.Context, savepointName, dbName string, tx sql.Transaction) error {
+	if !transactionsEnabled || dbName == "" {
+		return nil
+	}
+
+	dtx, ok := tx.(*DoltTransaction)
+	if !ok {
+		return fmt.Errorf("expected a DoltTransaction")
+	}
+
+	return dtx.CreateSavepoint(savepointName, sess.roots[dbName].root)
+}
+
+// RollbackToSavepoint sets this session's root to the one saved in the savepoint name. It's an error if not savepoint
+// with that name exists.
+func (sess *DoltSession) RollbackToSavepoint(ctx *sql.Context, savepointName, dbName string, tx sql.Transaction) error {
+	if !transactionsEnabled || dbName == "" {
+		return nil
+	}
+
+	dtx, ok := tx.(*DoltTransaction)
+	if !ok {
+		return fmt.Errorf("expected a DoltTransaction")
+	}
+
+	root := dtx.GetSavepoint(savepointName)
+	if root == nil {
+		return sql.ErrSavepointDoesNotExist.New(savepointName)
+	}
+
+	return sess.SetRoot(ctx, dbName, root)
+}
+
+// ReleaseSavepoint removes the savepoint name from the transaction. It's an error if no savepoint with that name
+// exists.
+func (sess *DoltSession) ReleaseSavepoint(ctx *sql.Context, savepointName, dbName string, tx sql.Transaction) error {
+	if !transactionsEnabled || dbName == "" {
+		return nil
+	}
+
+	dtx, ok := tx.(*DoltTransaction)
+	if !ok {
+		return fmt.Errorf("expected a DoltTransaction")
+	}
+
+	root := dtx.ClearSavepoint(savepointName)
+	if root == nil {
+		return sql.ErrSavepointDoesNotExist.New(savepointName)
+	}
+
+	return nil
+}
+
 // GetDoltDB returns the *DoltDB for a given database by name
 func (sess *DoltSession) GetDoltDB(dbName string) (*doltdb.DoltDB, bool) {
 	d, ok := sess.dbDatas[dbName]
@@ -463,30 +530,52 @@ func (sess *DoltSession) SetSessionVarDirectly(ctx *sql.Context, key string, val
 
 // AddDB adds the database given to this session. This establishes a starting root value for this session, as well as
 // other state tracking metadata.
-func (sess *DoltSession) AddDB(ctx *sql.Context, db Database) error {
+func (sess *DoltSession) AddDB(ctx *sql.Context, db sql.Database, dbData env.DbData) error {
 	defineSystemVariables(db.Name())
 
-	rsr := db.GetStateReader()
-	rsw := db.GetStateWriter()
-	drw := db.GetDocsReadWriter()
-	ddb := db.GetDoltDB()
+	rsr := dbData.Rsr
+	ddb := dbData.Ddb
 
-	sess.dbDatas[db.Name()] = env.DbData{Drw: drw, Rsr: rsr, Rsw: rsw, Ddb: ddb}
+	sess.dbDatas[db.Name()] = dbData
 	sess.editSessions[db.Name()] = editor.CreateTableEditSession(nil, editor.TableEditSessionProps{})
-	sess.caches[db.name] = newTableCache()
 
 	cs := rsr.CWBHeadSpec()
 	headRef := rsr.CWBHeadRef()
 
-	var workingRoot *doltdb.RootValue
 	workingHashInRepoState := rsr.WorkingHash()
 	workingHashInWsRef := hash.Hash{}
 
-	workingRoot, err := ddb.ReadRootValue(ctx, workingHashInRepoState)
+	// TODO: this resolve isn't necessary in all cases and slows things down
+	cm, err := ddb.Resolve(ctx, cs, headRef)
 	if err != nil {
 		return err
 	}
 
+	headCommitHash, err := cm.HashOf()
+	if err != nil {
+		return err
+	}
+
+	var workingRoot *doltdb.RootValue
+	// Get a working root to use for this session. This could come from the an independent working set not associated
+	// with any commit, or from the head commit itself in some use cases. Some implementors of RepoStateReader use the
+	// current HEAD hash as the working set hash, and in fact they have to -- there's not always an independently
+	// addressable root value available, only one persisted as a value in a Commit object.
+	if headCommitHash == workingHashInRepoState {
+		workingRoot, err = cm.GetRootValue()
+		if err != nil {
+			return err
+		}
+	}
+
+	if workingRoot == nil {
+		// If the root isn't a head commit value, assume it's a standalone value and look it up
+		workingRoot, err = ddb.ReadRootValue(ctx, workingHashInRepoState)
+		if err != nil {
+			return err
+		}
+	}
+
 	if transactionsEnabled {
 		// Not all dolt commands update the working set ref yet. So until that's true, we update it here with the contents
 		// of the repo_state.json file
@@ -494,7 +583,7 @@ func (sess *DoltSession) AddDB(ctx *sql.Context, db Database) error {
 		if err != nil {
 			return err
 		}
-		sess.workingSets[db.name] = workingSetRef
+		sess.workingSets[db.Name()] = workingSetRef
 
 		workingSet, err := ddb.ResolveWorkingSet(ctx, workingSetRef)
 		if err == doltdb.ErrWorkingSetNotFound {
@@ -520,18 +609,7 @@ func (sess *DoltSession) AddDB(ctx *sql.Context, db Database) error {
 		}
 	}
 
-	err = sess.SetRoot(ctx, db.name, workingRoot)
-	if err != nil {
-		return err
-	}
-
-	// TODO: this resolve probably isn't necessary and slows things down
-	cm, err := ddb.Resolve(ctx, cs, headRef)
-	if err != nil {
-		return err
-	}
-
-	headCommitHash, err := cm.HashOf()
+	err = sess.SetRoot(ctx, db.Name(), workingRoot)
 	if err != nil {
 		return err
 	}
