@@ -72,19 +72,18 @@ type projected interface {
 
 // DoltTable implements the sql.Table interface and gives access to dolt table rows and schema.
 type DoltTable struct {
-	name   string
-	sqlSch sql.Schema
-	db     SqlDatabase
-
-	nbf        *types.NomsBinFormat
-	table      *doltdb.Table
-	sch        schema.Schema
-	autoIncCol schema.Column
+	tableName    string
+	sqlSch       sql.Schema
+	db           SqlDatabase
+	lockedToRoot *doltdb.RootValue
+	nbf          *types.NomsBinFormat
+	sch          schema.Schema
+	autoIncCol   schema.Column
 
 	projectedCols []string
 }
 
-func NewDoltTable(name string, sch schema.Schema, tbl *doltdb.Table, db SqlDatabase) DoltTable {
+func NewDoltTable(name string, sch schema.Schema, tbl *doltdb.Table, db SqlDatabase) *DoltTable {
 	var autoCol schema.Column
 	_ = sch.GetAllCols().Iter(func(tag uint64, col schema.Column) (stop bool, err error) {
 		if col.AutoIncrement {
@@ -94,10 +93,9 @@ func NewDoltTable(name string, sch schema.Schema, tbl *doltdb.Table, db SqlDatab
 		return
 	})
 
-	return DoltTable{
-		name:          name,
+	return &DoltTable{
+		tableName:     name,
 		db:            db,
-		table:         tbl,
 		nbf:           tbl.Format(),
 		sch:           sch,
 		autoIncCol:    autoCol,
@@ -105,6 +103,14 @@ func NewDoltTable(name string, sch schema.Schema, tbl *doltdb.Table, db SqlDatab
 	}
 }
 
+// LockedToRoot returns a version of this table with its root value locked to the given value. The table's values will
+// not change as the session's root value changes. Appropriate for AS OF queries, or other use cases where the table's
+// values should not change throughout execution of a session.
+func (t DoltTable) LockedToRoot(rootValue *doltdb.RootValue) *DoltTable {
+	t.lockedToRoot = rootValue
+	return &t
+}
+
 var _ sql.Table = (*DoltTable)(nil)
 var _ sql.IndexedTable = (*DoltTable)(nil)
 var _ sql.ForeignKeyTable = (*DoltTable)(nil)
@@ -128,9 +134,34 @@ func (t *DoltTable) WithIndexLookup(lookup sql.IndexLookup) sql.Table {
 	}
 }
 
+// doltTable returns the underlying doltTable from the current session
+func (t *DoltTable) doltTable(ctx *sql.Context) (*doltdb.Table, error) {
+	root := t.lockedToRoot
+	if root == nil {
+		var err error
+		root, err = t.db.GetRoot(ctx)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	table, ok, err := root.GetTable(ctx, t.tableName)
+	if err != nil {
+		return nil, err
+	}
+	if !ok {
+		return nil, fmt.Errorf("table not found: %s", t.tableName)
+	}
+
+	return table, nil
+}
+
 // GetIndexes implements sql.IndexedTable
 func (t *DoltTable) GetIndexes(ctx *sql.Context) ([]sql.Index, error) {
-	tbl := t.table
+	tbl, err := t.doltTable(ctx)
+	if err != nil {
+		return nil, err
+	}
 
 	sch, err := tbl.GetSchema(ctx)
 	if err != nil {
@@ -223,7 +254,12 @@ func (t *DoltTable) GetIndexes(ctx *sql.Context) ([]sql.Index, error) {
 
 // GetAutoIncrementValue gets the last AUTO_INCREMENT value
 func (t *DoltTable) GetAutoIncrementValue(ctx *sql.Context) (interface{}, error) {
-	val, err := t.table.GetAutoIncrementValue(ctx)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	val, err := table.GetAutoIncrementValue(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -232,18 +268,22 @@ func (t *DoltTable) GetAutoIncrementValue(ctx *sql.Context) (interface{}, error)
 
 // Name returns the name of the table.
 func (t *DoltTable) Name() string {
-	return t.name
+	return t.tableName
 }
 
 // String returns a human-readable string to display the name of this SQL node.
 func (t *DoltTable) String() string {
-	return t.name
+	return t.tableName
 }
 
 // NumRows returns the unfiltered count of rows contained in the table
 func (t *DoltTable) NumRows(ctx *sql.Context) (uint64, error) {
-	m, err := t.table.GetRowData(ctx)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return 0, err
+	}
 
+	m, err := table.GetRowData(ctx)
 	if err != nil {
 		return 0, err
 	}
@@ -267,7 +307,7 @@ func (t *DoltTable) sqlSchema() sql.Schema {
 	}
 
 	// TODO: fix panics
-	sqlSch, err := sqlutil.FromDoltSchema(t.name, t.sch)
+	sqlSch, err := sqlutil.FromDoltSchema(t.tableName, t.sch)
 	if err != nil {
 		panic(err)
 	}
@@ -278,7 +318,12 @@ func (t *DoltTable) sqlSchema() sql.Schema {
 
 // Returns the partitions for this table.
 func (t *DoltTable) Partitions(ctx *sql.Context) (sql.PartitionIter, error) {
-	rowData, err := t.table.GetRowData(ctx)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	rowData, err := table.GetRowData(ctx)
 
 	if err != nil {
 		return nil, err
@@ -357,12 +402,17 @@ func (itr emptyRowIterator) Close(*sql.Context) error {
 	return nil
 }
 
-// Returns the table rows for the partition given
+// PartitionRows returns the table rows for the partition given
 func (t *DoltTable) PartitionRows(ctx *sql.Context, partition sql.Partition) (sql.RowIter, error) {
-	return partitionRows(ctx, t, t.projectedCols, partition)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	return partitionRows(ctx, table, t.projectedCols, partition)
 }
 
-func partitionRows(ctx *sql.Context, t *DoltTable, projCols []string, partition sql.Partition) (sql.RowIter, error) {
+func partitionRows(ctx *sql.Context, t *doltdb.Table, projCols []string, partition sql.Partition) (sql.RowIter, error) {
 	switch typedPartition := partition.(type) {
 	case doltTablePartition:
 		if typedPartition.end == 0 {
@@ -379,7 +429,7 @@ func partitionRows(ctx *sql.Context, t *DoltTable, projCols []string, partition
 
 // WritableDoltTable allows updating, deleting, and inserting new rows. It implements sql.UpdatableTable and friends.
 type WritableDoltTable struct {
-	DoltTable
+	*DoltTable
 	db Database
 	ed *sqlTableEditor
 }
@@ -405,7 +455,7 @@ func (t *WritableDoltTable) WithIndexLookup(lookup sql.IndexLookup) sql.Table {
 
 func (t *WritableDoltTable) WithProjection(colNames []string) sql.Table {
 	return &WritableDoltTable{
-		DoltTable: *t.DoltTable.WithProjection(colNames).(*DoltTable),
+		DoltTable: t.DoltTable.WithProjection(colNames).(*DoltTable),
 		db:        t.db,
 		ed:        t.ed,
 	}
@@ -461,21 +511,26 @@ func (t *WritableDoltTable) Replacer(ctx *sql.Context) sql.RowReplacer {
 
 // Truncate implements sql.TruncateableTable
 func (t *WritableDoltTable) Truncate(ctx *sql.Context) (int, error) {
-	rowData, err := t.table.GetRowData(ctx)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return 0, err
+	}
+
+	rowData, err := table.GetRowData(ctx)
 	if err != nil {
 		return 0, err
 	}
 	numOfRows := int(rowData.Len())
-	schVal, err := encoding.MarshalSchemaAsNomsValue(ctx, t.table.ValueReadWriter(), t.sch)
+	schVal, err := encoding.MarshalSchemaAsNomsValue(ctx, table.ValueReadWriter(), t.sch)
 	if err != nil {
 		return 0, err
 	}
-	empty, err := types.NewMap(ctx, t.table.ValueReadWriter())
+	empty, err := types.NewMap(ctx, table.ValueReadWriter())
 	if err != nil {
 		return 0, err
 	}
 	// truncate table resets auto-increment value
-	newTable, err := doltdb.NewTable(ctx, t.table.ValueReadWriter(), schVal, empty, empty, nil)
+	newTable, err := doltdb.NewTable(ctx, table.ValueReadWriter(), schVal, empty, empty, nil)
 	if err != nil {
 		return 0, err
 	}
@@ -488,7 +543,7 @@ func (t *WritableDoltTable) Truncate(ctx *sql.Context) (int, error) {
 	if err != nil {
 		return 0, err
 	}
-	newRoot, err := root.PutTable(ctx, t.name, newTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, newTable)
 	if err != nil {
 		return 0, err
 	}
@@ -496,7 +551,7 @@ func (t *WritableDoltTable) Truncate(ctx *sql.Context) (int, error) {
 	if err != nil {
 		return 0, err
 	}
-	t.table = newTable
+
 	return numOfRows, nil
 }
 
@@ -530,7 +585,12 @@ func (t *WritableDoltTable) GetAutoIncrementValue(ctx *sql.Context) (interface{}
 }
 
 func (t *WritableDoltTable) GetChecks(ctx *sql.Context) ([]sql.CheckDefinition, error) {
-	sch, err := t.table.GetSchema(ctx)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return nil, err
+	}
+
+	sch, err := table.GetSchema(ctx)
 	if err != nil {
 		return nil, err
 	}
@@ -559,7 +619,7 @@ func (t *DoltTable) GetForeignKeys(ctx *sql.Context) ([]sql.ForeignKeyConstraint
 		return nil, err
 	}
 
-	declaredFk, _ := fkc.KeysForTable(t.name)
+	declaredFk, _ := fkc.KeysForTable(t.tableName)
 	toReturn := make([]sql.ForeignKeyConstraint, len(declaredFk))
 
 	for i, fk := range declaredFk {
@@ -590,15 +650,9 @@ func (t *DoltTable) Projection() []string {
 	return t.projectedCols
 }
 
-func (t *DoltTable) WithProjection(colNames []string) sql.Table {
-	return &DoltTable{
-		name:          t.name,
-		db:            t.db,
-		table:         t.table,
-		sch:           t.sch,
-		autoIncCol:    t.autoIncCol,
-		projectedCols: colNames,
-	}
+func (t DoltTable) WithProjection(colNames []string) sql.Table {
+	t.projectedCols = colNames
+	return &t
 }
 
 var _ sql.PartitionIter = (*doltTablePartitionIter)(nil)
@@ -708,7 +762,7 @@ func (t *AlterableDoltTable) AddColumn(ctx *sql.Context, column *sql.Column, ord
 		return err
 	}
 
-	table, _, err := root.GetTable(ctx, t.name)
+	table, _, err := root.GetTable(ctx, t.tableName)
 	if err != nil {
 		return err
 	}
@@ -717,7 +771,7 @@ func (t *AlterableDoltTable) AddColumn(ctx *sql.Context, column *sql.Column, ord
 	if err != nil {
 		return err
 	}
-	tags, err := root.GenerateTagsForNewColumns(ctx, t.name, []string{column.Name}, []types.NomsKind{ti.NomsKind()})
+	tags, err := root.GenerateTagsForNewColumns(ctx, t.tableName, []string{column.Name}, []types.NomsKind{ti.NomsKind()}, nil)
 	if err != nil {
 		return err
 	}
@@ -736,12 +790,12 @@ func (t *AlterableDoltTable) AddColumn(ctx *sql.Context, column *sql.Column, ord
 		nullable = alterschema.Null
 	}
 
-	updatedTable, err := alterschema.AddColumnToTable(ctx, root, table, t.name, col.Tag, col.Name, col.TypeInfo, nullable, col.Default, col.Comment, orderToOrder(order))
+	updatedTable, err := alterschema.AddColumnToTable(ctx, root, table, t.tableName, col.Tag, col.Name, col.TypeInfo, nullable, col.Default, col.Comment, orderToOrder(order))
 	if err != nil {
 		return err
 	}
 
-	newRoot, err := root.PutTable(ctx, t.name, updatedTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, updatedTable)
 	if err != nil {
 		return err
 	}
@@ -766,7 +820,7 @@ func (t *AlterableDoltTable) DropColumn(ctx *sql.Context, columnName string) err
 		return err
 	}
 
-	updatedTable, _, err := root.GetTable(ctx, t.name)
+	updatedTable, _, err := root.GetTable(ctx, t.tableName)
 	if err != nil {
 		return err
 	}
@@ -796,14 +850,14 @@ func (t *AlterableDoltTable) DropColumn(ctx *sql.Context, columnName string) err
 	if err != nil {
 		return err
 	}
-	declaresFk, referencesFk := fkCollection.KeysForTable(t.name)
+	declaresFk, referencesFk := fkCollection.KeysForTable(t.tableName)
 
 	updatedTable, err = alterschema.DropColumn(ctx, updatedTable, columnName, append(declaresFk, referencesFk...))
 	if err != nil {
 		return err
 	}
 
-	newRoot, err := root.PutTable(ctx, t.name, updatedTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, updatedTable)
 	if err != nil {
 		return err
 	}
@@ -819,7 +873,7 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 		return err
 	}
 
-	table, _, err := root.GetTable(ctx, t.name)
+	table, _, err := root.GetTable(ctx, t.tableName)
 	if err != nil {
 		return err
 	}
@@ -843,7 +897,7 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 	if err != nil {
 		return err
 	}
-	declaresFk, referencedByFk := fkCollection.KeysForTable(t.name)
+	declaresFk, referencedByFk := fkCollection.KeysForTable(t.tableName)
 	for _, foreignKey := range declaresFk {
 		if (foreignKey.OnUpdate == doltdb.ForeignKeyReferenceOption_SetNull || foreignKey.OnDelete == doltdb.ForeignKeyReferenceOption_SetNull) &&
 			col.IsNullable() {
@@ -867,7 +921,7 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 			}
 		}
 		if existingCol.Kind != col.Kind { // We only change the tag when the underlying Noms kind changes
-			tags, err := root.GenerateTagsForNewColumns(ctx, t.name, []string{col.Name}, []types.NomsKind{col.Kind})
+			tags, err := root.GenerateTagsForNewColumns(ctx, t.tableName, []string{col.Name}, []types.NomsKind{col.Kind}, nil)
 			if err != nil {
 				return err
 			}
@@ -883,7 +937,7 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 		return err
 	}
 
-	newRoot, err := root.PutTable(ctx, t.name, updatedTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, updatedTable)
 	if err != nil {
 		return err
 	}
@@ -892,12 +946,24 @@ func (t *AlterableDoltTable) ModifyColumn(ctx *sql.Context, columnName string, c
 }
 
 // CreateIndex implements sql.IndexAlterableTable
-func (t *AlterableDoltTable) CreateIndex(ctx *sql.Context, indexName string, using sql.IndexUsing, constraint sql.IndexConstraint, columns []sql.IndexColumn, comment string) error {
+func (t *AlterableDoltTable) CreateIndex(
+	ctx *sql.Context,
+	indexName string,
+	using sql.IndexUsing,
+	constraint sql.IndexConstraint,
+	columns []sql.IndexColumn,
+	comment string,
+) error {
 	if schema.IsKeyless(t.sch) {
 		return fmt.Errorf("indexes on keyless tables are not supported")
 	}
 
-	ret, err := createIndexForTable(ctx, t.table, indexName, using, constraint, columns, true, comment)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return err
+	}
+
+	ret, err := createIndexForTable(ctx, table, indexName, using, constraint, columns, true, comment)
 	if err != nil {
 		return err
 	}
@@ -912,10 +978,10 @@ func (t *AlterableDoltTable) CreateIndex(ctx *sql.Context, indexName string, usi
 		}
 		for _, fk := range fkc.AllKeys() {
 			newFk := fk
-			if t.name == fk.TableName && fk.TableIndex == ret.oldIndex.Name() {
+			if t.tableName == fk.TableName && fk.TableIndex == ret.oldIndex.Name() {
 				newFk.TableIndex = ret.newIndex.Name()
 			}
-			if t.name == fk.ReferencedTableName && fk.ReferencedTableIndex == ret.oldIndex.Name() {
+			if t.tableName == fk.ReferencedTableName && fk.ReferencedTableIndex == ret.oldIndex.Name() {
 				newFk.ReferencedTableIndex = ret.newIndex.Name()
 			}
 			fkc.RemoveKeys(fk)
@@ -929,7 +995,7 @@ func (t *AlterableDoltTable) CreateIndex(ctx *sql.Context, indexName string, usi
 			return err
 		}
 	}
-	newRoot, err := root.PutTable(ctx, t.name, ret.newTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, ret.newTable)
 	if err != nil {
 		return err
 	}
@@ -954,7 +1020,7 @@ func (t *AlterableDoltTable) DropIndex(ctx *sql.Context, indexName string) error
 	if err != nil {
 		return err
 	}
-	ourKeys, referencingKeys := fkc.KeysForTable(t.name)
+	ourKeys, referencingKeys := fkc.KeysForTable(t.tableName)
 	for _, k := range ourKeys {
 		if k.TableIndex == indexName {
 			return fmt.Errorf("cannot drop index: %s is referenced by foreign key %s",
@@ -972,7 +1038,7 @@ func (t *AlterableDoltTable) DropIndex(ctx *sql.Context, indexName string) error
 	if err != nil {
 		return err
 	}
-	newRoot, err := root.PutTable(ctx, t.name, newTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, newTable)
 	if err != nil {
 		return err
 	}
@@ -990,7 +1056,13 @@ func (t *AlterableDoltTable) RenameIndex(ctx *sql.Context, fromIndexName string,
 	if err != nil {
 		return err
 	}
-	newTable, err := t.table.UpdateSchema(ctx, t.sch)
+
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return err
+	}
+
+	newTable, err := table.UpdateSchema(ctx, t.sch)
 	if err != nil {
 		return err
 	}
@@ -1003,7 +1075,7 @@ func (t *AlterableDoltTable) RenameIndex(ctx *sql.Context, fromIndexName string,
 	if err != nil {
 		return err
 	}
-	newRoot, err := root.PutTable(ctx, t.name, newTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, newTable)
 	if err != nil {
 		return err
 	}
@@ -1026,13 +1098,18 @@ func (t *AlterableDoltTable) CreateForeignKey(
 	if fkName != "" && !doltdb.IsValidForeignKeyName(fkName) {
 		return fmt.Errorf("invalid foreign key name `%s` as it must match the regular expression %s", fkName, doltdb.ForeignKeyNameRegexStr)
 	}
-	isSelfFk := strings.ToLower(t.name) == strings.ToLower(refTblName)
+	isSelfFk := strings.ToLower(t.tableName) == strings.ToLower(refTblName)
 	if isSelfFk {
 		if len(columns) > 1 {
 			return fmt.Errorf("support for self referential composite foreign keys is not yet implemented")
 		}
 	}
 
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return err
+	}
+
 	tblCols := make([]schema.Column, len(columns))
 	colTags := make([]uint64, len(columns))
 	sqlColNames := make([]sql.IndexColumn, len(columns))
@@ -1040,7 +1117,7 @@ func (t *AlterableDoltTable) CreateForeignKey(
 		tableCol, ok := t.sch.GetAllCols().GetByNameCaseInsensitive(col)
 		if !ok {
 			//TODO: fix go-mysql-server equivalent check, needs two vals
-			return fmt.Errorf("table `%s` does not have column `%s`", t.name, col)
+			return fmt.Errorf("table `%s` does not have column `%s`", t.tableName, col)
 		}
 		if (onUpdate == sql.ForeignKeyReferenceOption_SetNull || onDelete == sql.ForeignKeyReferenceOption_SetNull) &&
 			!tableCol.IsNullable() {
@@ -1062,7 +1139,7 @@ func (t *AlterableDoltTable) CreateForeignKey(
 	var ok bool
 	var refSch schema.Schema
 	if isSelfFk {
-		refTbl = t.table
+		refTbl = table
 		refSch = t.sch
 	} else {
 		refTbl, _, ok, err = root.GetTableInsensitive(ctx, refTblName)
@@ -1114,18 +1191,19 @@ func (t *AlterableDoltTable) CreateForeignKey(
 	tableIndex, ok := t.sch.Indexes().GetIndexByTags(colTags...)
 	if !ok {
 		// if child index doesn't exist, create it
-		ret, err := createIndexForTable(ctx, t.table, "", sql.IndexUsing_Default, sql.IndexConstraint_None, sqlColNames, false, "")
+		ret, err := createIndexForTable(ctx, table, "", sql.IndexUsing_Default, sql.IndexConstraint_None, sqlColNames, false, "")
 		if err != nil {
 			return err
 		}
-		t.table = ret.newTable
+
+		table = ret.newTable
 		tableIndex = ret.newIndex
-		root, err = root.PutTable(ctx, t.name, t.table)
+		root, err = root.PutTable(ctx, t.tableName, table)
 		if err != nil {
 			return err
 		}
 		if isSelfFk {
-			refTbl = t.table
+			refTbl = table
 		}
 	}
 
@@ -1162,7 +1240,7 @@ func (t *AlterableDoltTable) CreateForeignKey(
 	}
 	foreignKey := doltdb.ForeignKey{
 		Name:                   fkName,
-		TableName:              t.name,
+		TableName:              t.tableName,
 		TableIndex:             tableIndex.Name(),
 		TableColumns:           colTags,
 		ReferencedTableName:    refTblName,
@@ -1180,7 +1258,7 @@ func (t *AlterableDoltTable) CreateForeignKey(
 		return err
 	}
 
-	tableIndexData, err := t.table.GetIndexRowData(ctx, tableIndex.Name())
+	tableIndexData, err := table.GetIndexRowData(ctx, tableIndex.Name())
 	if err != nil {
 		return err
 	}
@@ -1411,7 +1489,13 @@ func (t *AlterableDoltTable) dropIndex(ctx *sql.Context, indexName string) (*dol
 	if err != nil {
 		return nil, nil, err
 	}
-	newTable, err := t.table.UpdateSchema(ctx, t.sch)
+
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return nil, nil, err
+	}
+
+	newTable, err := table.UpdateSchema(ctx, t.sch)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -1427,15 +1511,15 @@ func (t *AlterableDoltTable) dropIndex(ctx *sql.Context, indexName string) (*dol
 }
 
 func (t *AlterableDoltTable) updateFromRoot(ctx *sql.Context, root *doltdb.RootValue) error {
-	updatedTableSql, ok, err := t.db.getTable(ctx, root, t.name)
+	updatedTableSql, ok, err := t.db.getTable(ctx, root, t.tableName)
 	if err != nil {
 		return err
 	}
 	if !ok {
-		return fmt.Errorf("table `%s` cannot find itself", t.name)
+		return fmt.Errorf("table `%s` cannot find itself", t.tableName)
 	}
 	var updatedTable *AlterableDoltTable
-	if doltdb.HasDoltPrefix(t.name) && !doltdb.IsReadOnlySystemTable(t.name) {
+	if doltdb.HasDoltPrefix(t.tableName) && !doltdb.IsReadOnlySystemTable(t.tableName) {
 		updatedTable = &AlterableDoltTable{*updatedTableSql.(*WritableDoltTable)}
 	} else {
 		updatedTable = updatedTableSql.(*AlterableDoltTable)
@@ -1445,25 +1529,28 @@ func (t *AlterableDoltTable) updateFromRoot(ctx *sql.Context, root *doltdb.RootV
 }
 
 func (t *AlterableDoltTable) CreateCheck(ctx *sql.Context, check *sql.CheckDefinition) error {
-	sch, err := t.table.GetSchema(ctx)
-	if err != nil {
-		return err
-	}
+	sch := t.sch
 
 	check = &(*check)
 	if check.Name == "" {
+		var err error
 		check.Name, err = t.generateCheckName(ctx, check)
 		if err != nil {
 			return err
 		}
 	}
 
-	_, err = sch.Checks().AddCheck(check.Name, check.CheckExpression, check.Enforced)
+	_, err := sch.Checks().AddCheck(check.Name, check.CheckExpression, check.Enforced)
 	if err != nil {
 		return err
 	}
 
-	newTable, err := t.table.UpdateSchema(ctx, sch)
+	table, err := t.doltTable(ctx)
+	if err != nil {
+		return err
+	}
+
+	newTable, err := table.UpdateSchema(ctx, sch)
 	if err != nil {
 		return err
 	}
@@ -1473,7 +1560,7 @@ func (t *AlterableDoltTable) CreateCheck(ctx *sql.Context, check *sql.CheckDefin
 		return err
 	}
 
-	newRoot, err := root.PutTable(ctx, t.name, newTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, newTable)
 	if err != nil {
 		return err
 	}
@@ -1487,17 +1574,19 @@ func (t *AlterableDoltTable) CreateCheck(ctx *sql.Context, check *sql.CheckDefin
 }
 
 func (t *AlterableDoltTable) DropCheck(ctx *sql.Context, chName string) error {
-	sch, err := t.table.GetSchema(ctx)
+	sch := t.sch
+
+	err := sch.Checks().DropCheck(chName)
 	if err != nil {
 		return err
 	}
 
-	err = sch.Checks().DropCheck(chName)
+	table, err := t.doltTable(ctx)
 	if err != nil {
 		return err
 	}
 
-	newTable, err := t.table.UpdateSchema(ctx, sch)
+	newTable, err := table.UpdateSchema(ctx, sch)
 	if err != nil {
 		return err
 	}
@@ -1507,7 +1596,7 @@ func (t *AlterableDoltTable) DropCheck(ctx *sql.Context, chName string) error {
 		return err
 	}
 
-	newRoot, err := root.PutTable(ctx, t.name, newTable)
+	newRoot, err := root.PutTable(ctx, t.tableName, newTable)
 	if err != nil {
 		return err
 	}
