@@ -35,7 +35,7 @@ import (
 
 var (
 	tableEditorMaxOps uint64 = 16384
-	ErrDuplicatePK           = errors.New("duplicate key error")
+	ErrDuplicateKey          = errors.New("duplicate key error")
 )
 
 func init() {
@@ -46,7 +46,7 @@ func init() {
 	}
 }
 
-type PKDuplicateErrFunc func(keyString string, k, v types.Tuple) error
+type PKDuplicateErrFunc func(keyString string, k, v types.Tuple, isPk bool) error
 
 type TableEditor interface {
 	InsertKeyVal(ctx context.Context, key, val types.Tuple, tagToVal map[uint64]types.Value, errFunc PKDuplicateErrFunc) error
@@ -148,7 +148,7 @@ func newPkTableEditor(ctx context.Context, t *doltdb.Table, tableSch schema.Sche
 		if err != nil {
 			return nil, err
 		}
-		te.indexEds[i] = NewIndexEditor(index, indexData)
+		te.indexEds[i] = NewIndexEditor(ctx, index, indexData, tableSch)
 	}
 
 	err = tableSch.GetAllCols().Iter(func(tag uint64, col schema.Column) (stop bool, err error) {
@@ -236,19 +236,18 @@ func (tea *tableEditAccumulator) maybeGet(ctx context.Context, keyHash hash.Hash
 
 // ContainsIndexedKey returns whether the given key is contained within the index. The key is assumed to be in the
 // format expected of the index, similar to searching on the index map itself.
-func ContainsIndexedKey(ctx context.Context, te TableEditor, key types.Tuple, indexName string) (bool, error) {
+func ContainsIndexedKey(ctx context.Context, te TableEditor, key types.Tuple, indexName string, idxSch schema.Schema) (bool, error) {
 	tbl, err := te.Table(ctx)
 	if err != nil {
 		return false, err
 	}
 
-	idxSch := te.Schema().Indexes().GetByName(indexName)
 	idxMap, err := tbl.GetIndexRowData(ctx, indexName)
 	if err != nil {
 		return false, err
 	}
 
-	indexIter := noms.NewNomsRangeReader(idxSch.Schema(), idxMap,
+	indexIter := noms.NewNomsRangeReader(idxSch, idxMap,
 		[]*noms.ReadRange{{Start: key, Inclusive: true, Reverse: false, Check: func(tuple types.Tuple) (bool, error) {
 			return tuple.StartsWith(key), nil
 		}}},
@@ -266,19 +265,18 @@ func ContainsIndexedKey(ctx context.Context, te TableEditor, key types.Tuple, in
 
 // GetIndexedRows returns all matching rows for the given key on the index. The key is assumed to be in the format
 // expected of the index, similar to searching on the index map itself.
-func GetIndexedRows(ctx context.Context, te TableEditor, key types.Tuple, indexName string) ([]row.Row, error) {
+func GetIndexedRows(ctx context.Context, te TableEditor, key types.Tuple, indexName string, idxSch schema.Schema) ([]row.Row, error) {
 	tbl, err := te.Table(ctx)
 	if err != nil {
 		return nil, err
 	}
 
-	idxSch := te.Schema().Indexes().GetByName(indexName)
 	idxMap, err := tbl.GetIndexRowData(ctx, indexName)
 	if err != nil {
 		return nil, err
 	}
 
-	indexIter := noms.NewNomsRangeReader(idxSch.Schema(), idxMap,
+	indexIter := noms.NewNomsRangeReader(idxSch, idxMap,
 		[]*noms.ReadRange{{Start: key, Inclusive: true, Reverse: false, Check: func(tuple types.Tuple) (bool, error) {
 			return tuple.StartsWith(key), nil
 		}}},
@@ -333,7 +331,7 @@ func GetIndexedRows(ctx context.Context, te TableEditor, key types.Tuple, indexN
 	return rows, nil
 }
 
-func (te *pkTableEditor) pkErrForKVP(ctx context.Context, kvp *doltKVP, errFunc PKDuplicateErrFunc) error {
+func (te *pkTableEditor) keyErrForKVP(ctx context.Context, kvp *doltKVP, isPk bool, errFunc PKDuplicateErrFunc) error {
 	kVal, err := kvp.k.Value(ctx)
 	if err != nil {
 		return err
@@ -350,14 +348,14 @@ func (te *pkTableEditor) pkErrForKVP(ctx context.Context, kvp *doltKVP, errFunc
 	}
 
 	if errFunc != nil {
-		return errFunc(keyStr, kVal.(types.Tuple), vVal.(types.Tuple))
+		return errFunc(keyStr, kVal.(types.Tuple), vVal.(types.Tuple), isPk)
 	} else {
-		return fmt.Errorf("duplicate key '%s': %w", keyStr, ErrDuplicatePK)
+		return fmt.Errorf("duplicate key '%s': %w", keyStr, ErrDuplicateKey)
 	}
 }
 
-// TODO - Deduplicate this code.  It is largely a copy of insert
-func (te *pkTableEditor) InsertKeyVal(ctx context.Context, key, val types.Tuple, tagToVal map[uint64]types.Value, errFunc PKDuplicateErrFunc) error {
+// InsertKeyVal adds the given tuples to the table.
+func (te *pkTableEditor) InsertKeyVal(ctx context.Context, key, val types.Tuple, tagToVal map[uint64]types.Value, errFunc PKDuplicateErrFunc) (retErr error) {
 	defer te.autoFlush()
 	te.flushMutex.RLock()
 	defer te.flushMutex.RUnlock()
@@ -381,16 +379,57 @@ func (te *pkTableEditor) InsertKeyVal(ctx context.Context, key, val types.Tuple,
 	te.writeMutex.Lock()
 	defer te.writeMutex.Unlock()
 
+	// Run the index editors first, as we can back out of the changes in the event of an error, but can't do that for
+	// changes made to the table. We create a slice that matches the number of index editors. For each successful
+	// operation, we increment the associated index on the slice, and in the event of an error, we undo that number of
+	// operations.
+	indexOpsToUndo := make([]int, len(te.indexEds))
+	defer func() {
+		if retErr != nil {
+			for i, opsToUndo := range indexOpsToUndo {
+				for undone := 0; undone < opsToUndo; undone++ {
+					te.indexEds[i].Undo(ctx)
+				}
+			}
+		}
+	}()
+
+	for i, indexEd := range te.indexEds {
+		fullKey, partialKey, err := row.ReduceToIndexKeysFromTagMap(te.nbf, indexEd.Index(), tagToVal)
+		if err != nil {
+			return err
+		}
+		err = indexEd.InsertRow(ctx, fullKey, partialKey)
+		if uke, ok := err.(*uniqueKeyErr); ok {
+			tableTupleHash, err := uke.TableTuple.Hash(uke.TableTuple.Format())
+			if err != nil {
+				return err
+			}
+			kvp, pkExists, err := te.tea.maybeGet(ctx, tableTupleHash, uke.TableTuple)
+			if err != nil {
+				return err
+			}
+			if !pkExists {
+				keyStr, _ := formatKey(ctx, uke.TableTuple)
+				return fmt.Errorf("UNIQUE constraint violation on index '%s', but could not find row with primary key: %s",
+					indexEd.Index().Name(), keyStr)
+			}
+			return te.keyErrForKVP(ctx, kvp, false, errFunc)
+		} else if err != nil {
+			return err
+		}
+		indexOpsToUndo[i]++
+	}
+
 	if kvp, pkExists, err := te.tea.maybeGet(ctx, keyHash, key); err != nil {
 		return err
 	} else if pkExists {
-		return te.pkErrForKVP(ctx, kvp, errFunc)
+		return te.keyErrForKVP(ctx, kvp, true, errFunc)
 	}
 	te.tea.insertedKeys[keyHash] = key
 	te.tea.addedKeys[keyHash] = key
 
 	if te.hasAutoInc {
-		// autoIncVal = max(autoIncVal, insertVal)
 		insertVal, ok := tagToVal[te.autoIncCol.Tag]
 		if ok {
 			less, err := te.autoIncVal.Less(te.nbf, insertVal)
@@ -411,68 +450,35 @@ func (te *pkTableEditor) InsertKeyVal(ctx context.Context, key, val types.Tuple,
 	return nil
 }
 
-// InsertRow adds the given row to the table. If the row already exists, use UpdateRow.
+// InsertRow adds the given row to the table. If the row already exists, use UpdateRow. This converts the given row into
+// tuples that are then passed to InsertKeyVal.
 func (te *pkTableEditor) InsertRow(ctx context.Context, dRow row.Row, errFunc PKDuplicateErrFunc) error {
-	defer te.autoFlush()
-	te.flushMutex.RLock()
-	defer te.flushMutex.RUnlock()
-
 	key, err := dRow.NomsMapKey(te.tSch).Value(ctx)
 	if err != nil {
 		return err
 	}
-	keyHash, err := key.Hash(dRow.Format())
+	val, err := dRow.NomsMapValue(te.tSch).Value(ctx)
 	if err != nil {
 		return err
 	}
-
-	// We allow each write operation to perform as much work as possible before acquiring the lock. This minimizes the
-	// lock time and slightly increases throughput. Although this introduces variability in the amount of time before
-	// the lock is acquired, this is a non-issue, which is elaborated on using this example function.
-	// func Example(ctx context.Context, te *pkTableEditor, someRow row.Row) {
-	//     go te.Insert(ctx, someRow)
-	//     go te.Delete(ctx, someRow)
-	// }
-	// Let's pretend the table already has someRow. Go will run goroutines in any arbitrary order, sequentially or
-	// concurrently, and thus running Example() may see that Delete() executes before Insert(), causing a different
-	// result as Insert() executing first would result in an error. Such an issue must be handled above the pkTableEditor.
-	// Since we cannot guarantee any of that here, we can delay our lock acquisition.
-	te.writeMutex.Lock()
-	defer te.writeMutex.Unlock()
-
-	if kvp, pkExists, err := te.tea.maybeGet(ctx, keyHash, key); err != nil {
-		return err
-	} else if pkExists {
-		return te.pkErrForKVP(ctx, kvp, errFunc)
-	}
-	te.tea.insertedKeys[keyHash] = key
-	te.tea.addedKeys[keyHash] = key
-
-	if te.hasAutoInc {
-		// autoIncVal = max(autoIncVal, insertVal)
-		insertVal, ok := dRow.GetColVal(te.autoIncCol.Tag)
-		if ok {
-			less, err := te.autoIncVal.Less(te.nbf, insertVal)
-			if err != nil {
-				return err
-			}
-			if less {
-				te.autoIncVal = types.Round(insertVal)
-			}
-			te.autoIncVal = types.Increment(te.autoIncVal)
+	tagToVal := make(map[uint64]types.Value)
+	_, err = dRow.IterSchema(te.tSch, func(tag uint64, val types.Value) (stop bool, err error) {
+		if val == nil {
+			tagToVal[tag] = types.NullValue
+		} else {
+			tagToVal[tag] = val
 		}
+		return false, nil
+	})
+	if err != nil {
+		return err
 	}
-
-	val := dRow.NomsMapValue(te.tSch)
-	te.tea.ed.AddEdit(key, val)
-	te.tea.affectedKeys[keyHash] = &doltKVP{k: key, v: val}
-	te.tea.opCount++
-	return nil
+	return te.InsertKeyVal(ctx, key.(types.Tuple), val.(types.Tuple), tagToVal, errFunc)
 }
 
 // DeleteRow removes the given row from the table. This essentially acts as a convenience function for DeleteKey, while
 // ensuring proper thread safety.
-func (te *pkTableEditor) DeleteRow(ctx context.Context, dRow row.Row) error {
+func (te *pkTableEditor) DeleteRow(ctx context.Context, dRow row.Row) (retErr error) {
 	defer te.autoFlush()
 	te.flushMutex.RLock()
 	defer te.flushMutex.RUnlock()
@@ -482,23 +488,60 @@ func (te *pkTableEditor) DeleteRow(ctx context.Context, dRow row.Row) error {
 		return err
 	}
 
-	return te.delete(key.(types.Tuple))
-}
-
-// UpdateRow takes the current row and new rows, and updates it accordingly.
-func (te *pkTableEditor) UpdateRow(ctx context.Context, dOldRow row.Row, dNewRow row.Row, errFunc PKDuplicateErrFunc) error {
-	defer te.autoFlush()
-	te.flushMutex.RLock()
-	defer te.flushMutex.RUnlock()
-
-	dOldKey := dOldRow.NomsMapKey(te.tSch)
-	dOldKeyVal, err := dOldKey.Value(ctx)
+	keyHash, err := key.Hash(te.nbf)
 	if err != nil {
 		return err
 	}
 
-	dNewKey := dNewRow.NomsMapKey(te.tSch)
-	dNewKeyVal, err := dNewKey.Value(ctx)
+	// Regarding the lock's position here, refer to the comment in InsertKeyVal
+	te.writeMutex.Lock()
+	defer te.writeMutex.Unlock()
+
+	// Index operations should come before all table operations. For the reasoning, refer to the comment in InsertKeyVal
+	indexOpsToUndo := make([]int, len(te.indexEds))
+	defer func() {
+		if retErr != nil {
+			for i, opsToUndo := range indexOpsToUndo {
+				for undone := 0; undone < opsToUndo; undone++ {
+					te.indexEds[i].Undo(ctx)
+				}
+			}
+		}
+	}()
+
+	for i, indexEd := range te.indexEds {
+		fullKey, partialKey, err := row.ReduceToIndexKeys(indexEd.Index(), dRow)
+		if err != nil {
+			return err
+		}
+		err = indexEd.DeleteRow(ctx, fullKey, partialKey)
+		if err != nil {
+			return err
+		}
+		indexOpsToUndo[i]++
+	}
+
+	delete(te.tea.addedKeys, keyHash)
+	te.tea.removedKeys[keyHash] = key
+	te.tea.affectedKeys[keyHash] = &doltKVP{k: key}
+
+	te.tea.ed.AddEdit(key, nil)
+	te.tea.opCount++
+	return nil
+}
+
+// UpdateRow takes the current row and new rows, and updates it accordingly.
+func (te *pkTableEditor) UpdateRow(ctx context.Context, dOldRow row.Row, dNewRow row.Row, errFunc PKDuplicateErrFunc) (retErr error) {
+	defer te.autoFlush()
+	te.flushMutex.RLock()
+	defer te.flushMutex.RUnlock()
+
+	dOldKeyVal, err := dOldRow.NomsMapKey(te.tSch).Value(ctx)
+	if err != nil {
+		return err
+	}
+
+	dNewKeyVal, err := dNewRow.NomsMapKey(te.tSch).Value(ctx)
 	if err != nil {
 		return err
 	}
@@ -509,10 +552,58 @@ func (te *pkTableEditor) UpdateRow(ctx context.Context, dOldRow row.Row, dNewRow
 	}
 	oldKeyEqualsNewKey := dOldKeyVal.Equals(dNewKeyVal)
 
-	// Regarding the lock's position here, refer to the comment in InsertRow
+	// Regarding the lock's position here, refer to the comment in InsertKeyVal
 	te.writeMutex.Lock()
 	defer te.writeMutex.Unlock()
 
+	// Index operations should come before all table operations. For the reasoning, refer to the comment in InsertKeyVal
+	indexOpsToUndo := make([]int, len(te.indexEds))
+	defer func() {
+		if retErr != nil {
+			for i, opsToUndo := range indexOpsToUndo {
+				for undone := 0; undone < opsToUndo; undone++ {
+					te.indexEds[i].Undo(ctx)
+				}
+			}
+		}
+	}()
+
+	for i, indexEd := range te.indexEds {
+		oldFullKey, oldPartialKey, err := row.ReduceToIndexKeys(indexEd.Index(), dOldRow)
+		if err != nil {
+			return err
+		}
+		err = indexEd.DeleteRow(ctx, oldFullKey, oldPartialKey)
+		if err != nil {
+			return err
+		}
+		indexOpsToUndo[i]++
+		newFullKey, newPartialKey, err := row.ReduceToIndexKeys(indexEd.Index(), dNewRow)
+		if err != nil {
+			return err
+		}
+		err = indexEd.InsertRow(ctx, newFullKey, newPartialKey)
+		if uke, ok := err.(*uniqueKeyErr); ok {
+			tableTupleHash, err := uke.TableTuple.Hash(uke.TableTuple.Format())
+			if err != nil {
+				return err
+			}
+			kvp, pkExists, err := te.tea.maybeGet(ctx, tableTupleHash, uke.TableTuple)
+			if err != nil {
+				return err
+			}
+			if !pkExists {
+				keyStr, _ := formatKey(ctx, uke.TableTuple)
+				return fmt.Errorf("UNIQUE constraint violation on index '%s', but could not find row with primary key: %s",
+					indexEd.Index().Name(), keyStr)
+			}
+			return te.keyErrForKVP(ctx, kvp, false, errFunc)
+		} else if err != nil {
+			return err
+		}
+		indexOpsToUndo[i]++
+	}
+
 	// If the PK is changed then we need to delete the old value and insert the new one
 	if !oldKeyEqualsNewKey {
 		oldHash, err := dOldKeyVal.Hash(dOldRow.Format())
@@ -531,7 +622,7 @@ func (te *pkTableEditor) UpdateRow(ctx context.Context, dOldRow row.Row, dNewRow
 		if kvp, pkExists, err := te.tea.maybeGet(ctx, newHash, dNewKeyVal); err != nil {
 			return err
 		} else if pkExists {
-			return te.pkErrForKVP(ctx, kvp, errFunc)
+			return te.keyErrForKVP(ctx, kvp, true, errFunc)
 		}
 		te.tea.addedKeys[newHash] = dNewKeyVal
 		te.tea.removedKeys[oldHash] = dOldKeyVal
@@ -560,9 +651,15 @@ func (te *pkTableEditor) SetAutoIncrementValue(v types.Value) (err error) {
 }
 
 // Table returns a Table based on the edits given, if any. If Flush() was not called prior, it will be called here.
-func (te *pkTableEditor) Table(_ context.Context) (*doltdb.Table, error) {
+func (te *pkTableEditor) Table(ctx context.Context) (*doltdb.Table, error) {
 	te.flush()
 	err := te.aq.WaitForEmpty()
+	if err != nil {
+		return nil, err
+	}
+
+	te.flushMutex.Lock()
+	defer te.flushMutex.Unlock()
 
 	if te.hasAutoInc {
 		te.t, err = te.t.SetAutoIncrementValue(te.autoIncVal)
@@ -571,7 +668,37 @@ func (te *pkTableEditor) Table(_ context.Context) (*doltdb.Table, error) {
 		}
 	}
 
-	return te.t, err
+	tbl := te.t
+	idxMutex := &sync.Mutex{}
+	idxWg := &sync.WaitGroup{}
+	idxWg.Add(len(te.indexEds))
+	for i := 0; i < len(te.indexEds); i++ {
+		go func(i int) {
+			defer idxWg.Done()
+			indexMap, idxErr := te.indexEds[i].Map(ctx)
+			idxMutex.Lock()
+			defer idxMutex.Unlock()
+			if err != nil {
+				return
+			}
+			if idxErr != nil {
+				err = idxErr
+				return
+			}
+			tbl, idxErr = tbl.SetIndexRowData(ctx, te.indexEds[i].Index().Name(), indexMap)
+			if idxErr != nil {
+				err = idxErr
+				return
+			}
+		}(i)
+	}
+	idxWg.Wait()
+	if err != nil {
+		return nil, err
+	}
+	te.t = tbl
+
+	return te.t, nil
 }
 
 func (te *pkTableEditor) Schema() schema.Schema {
@@ -591,7 +718,10 @@ func (te *pkTableEditor) Format() *types.NomsBinFormat {
 func (te *pkTableEditor) Close() error {
 	te.tea.ed.Close()
 	for _, indexEd := range te.indexEds {
-		indexEd.ed.Close()
+		err := indexEd.Close()
+		if err != nil {
+			return err
+		}
 	}
 	return nil
 }
@@ -622,25 +752,7 @@ func (te *pkTableEditor) autoFlush() {
 	}
 }
 
-func (te *pkTableEditor) delete(key types.Tuple) error {
-	keyHash, err := key.Hash(te.nbf)
-	if err != nil {
-		return err
-	}
-
-	te.writeMutex.Lock()
-	defer te.writeMutex.Unlock()
-
-	delete(te.tea.addedKeys, keyHash)
-	te.tea.removedKeys[keyHash] = key
-	te.tea.affectedKeys[keyHash] = &doltKVP{k: key}
-
-	te.tea.ed.AddEdit(key, nil)
-	te.tea.opCount++
-	return nil
-}
-
-func (te *pkTableEditor) flushEditAccumulator(ctx context.Context, teaInterface interface{}) error {
+func (te *pkTableEditor) flushEditAccumulator(ctx context.Context, teaInterface interface{}) (err error) {
 	// We don't call any locks at the function entrance since this is called from an ActionExecutor with a concurrency of 1
 	futureTea := teaInterface.(*tableEditAccumulator)
 	tea := futureTea.prevTea
@@ -656,7 +768,13 @@ func (te *pkTableEditor) flushEditAccumulator(ctx context.Context, teaInterface
 	// If we encounter an error and return, then we need to remove this tea from the chain and update the next's rowData
 	encounteredErr := true
 	defer func() {
+		//TODO: need some way to reset an index editor to a previous point as well
 		if encounteredErr {
+			// As this is in a defer and we're attempting to capture all errors, that includes panics as well.
+			// Naturally a panic doesn't set the err variable, so we have to recover it.
+			if recoveredErr := recover(); recoveredErr != nil && err == nil {
+				err = recoveredErr.(error)
+			}
 			// All tea modifications are guarded by writeMutex locks, so we have to acquire it
 			te.writeMutex.Lock()
 			futureTea.prevTea = nil
@@ -678,10 +796,6 @@ func (te *pkTableEditor) flushEditAccumulator(ctx context.Context, teaInterface
 	if err != nil {
 		return err
 	}
-	newTable, err = te.updateIndexes(ctx, tea, newTable, tea.rowData, updatedMap)
-	if err != nil {
-		return err
-	}
 	// No errors were encountered, so we set the bool to false. This should come after ALL calls that may error.
 	encounteredErr = false
 
@@ -729,84 +843,3 @@ func formatKey(ctx context.Context, key types.Value) (string, error) {
 
 	return fmt.Sprintf("[%s]", strings.Join(vals, ",")), nil
 }
-
-func (te *pkTableEditor) updateIndexes(ctx context.Context, tea *tableEditAccumulator, tbl *doltdb.Table, originalRowData types.Map, updated types.Map) (*doltdb.Table, error) {
-	// We don't call any locks here since this is called from an ActionExecutor with a concurrency of 1
-	if len(te.indexEds) == 0 {
-		return tbl, nil
-	}
-
-	indexActionQueue := async.NewActionExecutor(ctx, func(_ context.Context, keyInt interface{}) error {
-		key := keyInt.(types.Value)
-
-		var originalRow row.Row
-		var updatedRow row.Row
-
-		if val, ok, err := originalRowData.MaybeGet(ctx, key); err == nil && ok {
-			originalRow, err = row.FromNoms(te.tSch, key.(types.Tuple), val.(types.Tuple))
-			if err != nil {
-				return err
-			}
-		} else if err != nil {
-			return err
-		}
-		if val, ok, err := updated.MaybeGet(ctx, key); err == nil && ok {
-			updatedRow, err = row.FromNoms(te.tSch, key.(types.Tuple), val.(types.Tuple))
-			if err != nil {
-				return err
-			}
-		} else if err != nil {
-			return err
-		}
-
-		for _, indexEd := range te.indexEds {
-			var err error
-			var originalIndexRow row.Row
-			var updatedIndexRow row.Row
-			if originalRow != nil {
-				originalIndexRow, err = row.ReduceToIndex(indexEd.Index(), originalRow)
-				if err != nil {
-					return err
-				}
-			}
-			if updatedRow != nil {
-				updatedIndexRow, err = row.ReduceToIndex(indexEd.Index(), updatedRow)
-				if err != nil {
-					return err
-				}
-			}
-
-			err = indexEd.UpdateIndex(ctx, originalIndexRow, updatedIndexRow)
-			if err != nil {
-				return err
-			}
-		}
-
-		return nil
-	}, 4, 0)
-
-	for _, kvp := range tea.affectedKeys {
-		indexActionQueue.Execute(kvp.k)
-	}
-
-	err := indexActionQueue.WaitForEmpty()
-	if err != nil {
-		return nil, err
-	}
-
-	for _, indexEd := range te.indexEds {
-		if !indexEd.HasChanges() {
-			continue
-		}
-		indexMap, err := indexEd.Map(ctx)
-		if err != nil {
-			return nil, err
-		}
-		tbl, err = tbl.SetIndexRowData(ctx, indexEd.Index().Name(), indexMap)
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	return tbl, nil
-}
